#### Introduction ####

## How to use this code:
## 1. Set a working directory
## 2. Place the code and data files ("datasets.json")
## 3. Run the code
## 4. The output will appear in the directory as PDF files
## If you have any questions, feel free to contact me: Yiyuan Zou (y.zou@tudelft.nl)

#### Load Data and Libraries ####

# Libraries
library(jsonlite)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(ggExtra)
# Be careful with the order of "plyr" and "dplyr"
library(plyr)
library(dplyr)
library(tm)
library(wordcloud)
library(RColorBrewer)

# Load JSON data
datasets <- fromJSON("datasets.json")

# Flatten nested data frames
dataframe <- flatten(datasets, recursive = TRUE)

# Record the order of properties
property_order <- rev(colnames(dataframe))

# Define a list to store plots
plots <- list()

#### Data Completeness and Uniqueness ####

# Number of NA in each column
non_na_percent <- colSums(!is.na(dataframe) & dataframe != "")/nrow(dataframe)
non_na_percent_df <- data.frame(Property = names(non_na_percent), Percentage = non_na_percent) %>%
  mutate(Group = "Data Completeness Check")

# Number of unique elements in each column
unique_percent <- sapply(dataframe, function(x) length(unique(x))/nrow(dataframe))
unique_percent_df <- data.frame(Property = names(unique_percent), Percentage = unique_percent)%>%
  mutate(Group = "Data Uniqueness Check")

percent_df <- rbind(non_na_percent_df, unique_percent_df)
rownames(percent_df) <- NULL

# Barplot
plots[[1]] <- ggplot(percent_df, aes(x = factor(Property, property_order), y = Percentage))+
  geom_bar(fill = "#6499E9", alpha = 0.6, width = 1, stat = "identity")+
  facet_wrap(~ Group, ncol = 2)+
  theme_minimal()+ 
  coord_flip()+
  labs(x = "Property",
       y = "Percentage of non-null data (left) and unique data (right)")+
  theme(axis.title = element_text(size = 12),
        strip.text = element_text(size = 12),
        axis.text = element_text(size = 10))+
  scale_y_continuous(labels = scales::label_percent(accuracy = 1))

# Remove embargo-related data
dataframe <- dataframe[!is.na(dataframe$uuid),]

# Remove completely duplicate data
dataframe = unique(dataframe)

#### Timeline: Heatmap (published_date) ####

# Extract years and months
dataframe <- dataframe %>% 
  mutate(published_year = substr(dataframe$published_date, 1,4),
         published_month = substr(dataframe$published_date, 6,7))

# First Online
counts <- ddply(dataframe, .(Year = dataframe$published_year, Month = dataframe$published_month), nrow)
counts <- na.omit(counts)

# Heatmap 
gg_hm <- ggplot(counts, aes(Year, Month, fill= V1)) +
  geom_tile(color = "black")+
  geom_text(aes(label = V1), color = "black", size = 3.5)+
  scale_fill_gradientn(colours = c("#35A29F", "#FFFFCC","#F39F5A"),
                       values = c(0, 0.1, 1), limits = c(0, 800))+
  coord_fixed()+
  theme(legend.position = "bottom",
        legend.key.width = unit(0.6, "cm"),
        axis.title = element_text(size = 12),
        axis.text = element_text(size = 10))+
  labs(x = "Publication Year", y = "Publication Month", fill = 'Count') 

# Extract the legend of the heatmap
gg_legend <- as_ggplot(get_legend(gg_hm))
gg_hm <- gg_hm + theme(legend.position = "none")

gg_rows <- counts %>% 
  group_by(Year) %>% 
  summarize(sum_value = sum(V1)) %>% 
  ggplot(aes(x = Year, y = sum_value)) + 
  geom_bar(fill = "#6499E9", alpha = 0.6, stat = "identity", width = 0.8)+
  theme_minimal()+
  theme(axis.text.x = element_blank(),
        axis.title.x = element_blank(),
        axis.text = element_text(size = 10))+
  ylab("Count")

gg_cols <- counts %>% 
  group_by(Month) %>% 
  summarize(sum_value = sum(V1)) %>% 
  ggplot(aes(x = Month, y = sum_value))+ 
  geom_bar(fill = "#6499E9", alpha = 0.6, stat = "identity", width = 0.8)+
  theme_minimal()+
  coord_flip()+
  theme(axis.text.y = element_blank(),
        axis.title.y = element_blank(),
        axis.text = element_text(size = 10))+
  ylab("Count")

gg_empty <- counts %>% 
  ggplot(aes(x = Month, y = V1)) +
  geom_blank() +
  theme(axis.text = element_blank(),
        axis.title = element_blank(),
        line = element_blank(),
        panel.background = element_blank())

plots[[2]] <- egg:: ggarrange(
  gg_rows, gg_legend, gg_hm, gg_cols,
  nrow = 2, ncol = 2, widths = c(3, 1), heights = c(1, 3)
)

#### Save all the plots to one pdf file ####
pdf(file= "output.pdf", width = 9, height = 7)
plots[[1]]
plots[[2]]

#### Wordcloud: Title and Group_id ####

title <- dataframe$title

# Create a corpus  
docs <- Corpus(VectorSource(title))

# Clean the text
docs <- docs %>%
  tm_map(removeNumbers) %>%
  tm_map(removePunctuation) %>%
  tm_map(stripWhitespace)

# To lowercase
docs <- tm_map(docs, content_transformer(tolower))

# Remove stop words
docs <- tm_map(docs, removeWords, stopwords("english"))

# Remove specific words
# docs <- tm_map(docs, removeWords, c("data", "dataset"))

# Create a document-term-matrix
dtm <- TermDocumentMatrix(docs) 
matrix <- as.matrix(dtm) 
words <- sort(rowSums(matrix),decreasing = TRUE) 
words_df <- data.frame(word = names(words),freq = words)

set.seed(123) # for reproducibility 
# Set plot parameters
wordcloud(words = words_df$word, freq = words_df$freq, scale = c(4, 0.8), 
          min.freq = 10, max.words = 200, random.order = FALSE, rot.per = 0.35,
          colors = brewer.pal(8, "Dark2"))
title("Word Cloud of Titles in the Dataset")

# Most frequent words
frequent_words <- data.frame(Word = words_df[1:30,]$word, Frequency = words_df[1:30,]$freq)
ggplot(frequent_words, aes(x = factor(Word, rev(Word)), y = Frequency))+
  geom_bar(fill = "#6499E9", alpha = 0.6, width = 0.8, stat = "identity")+
  theme_minimal()+ 
  coord_flip()+
  labs(x = "Frequent Words",
       y = "Word Frequency")+
  theme(axis.title = element_text(size = 12),
        strip.text = element_text(size = 12),
        axis.text = element_text(size = 10))
dev.off()